@@ -15,12 +15,13 @@ from secretstorage.collection import Collection, create_collection, \
  get_all_collections, get_default_collection, get_any_collection, \
  get_collection_by_alias, search_items
 from secretstorage.item import Item
-from secretstorage.defines import DBUS_NOT_SUPPORTED, DBUS_EXEC_FAILED
+from secretstorage.defines import DBUS_NOT_SUPPORTED, DBUS_EXEC_FAILED, \
+ DBUS_NO_REPLY
 from secretstorage.exceptions import SecretStorageException, \
  SecretServiceNotAvailableException, LockedException, \
  ItemNotFoundException
 
-__version__ = '1.1.0'
+__version__ = '2.0.0'
 
 def dbus_init(main_loop=True, use_qt_loop=False):
 	"""Returns new SessionBus_. If `main_loop` is :const:`True`, registers
@@ -43,7 +44,8 @@ def dbus_init(main_loop=True, use_qt_loop=False):
 	try:
 		return dbus.SessionBus()
 	except dbus.exceptions.DBusException as e:
-		if e.get_dbus_name() in (DBUS_NOT_SUPPORTED, DBUS_EXEC_FAILED):
+		if e.get_dbus_name() in (DBUS_NOT_SUPPORTED,
+		DBUS_EXEC_FAILED, DBUS_NO_REPLY):
 			raise SecretServiceNotAvailableException(
 				e.get_dbus_message())
 		raise
