@@ -7,9 +7,15 @@
 normally be used by external applications."""
 
 import dbus
-from secretstorage.defines import *
+from secretstorage.defines import DBUS_UNKNOWN_METHOD, DBUS_NO_SUCH_OBJECT, \
+ DBUS_SERVICE_UNKNOWN, DBUS_NO_REPLY, DBUS_NOT_SUPPORTED, DBUS_EXEC_FAILED, \
+ SECRETS, SS_PATH, SS_PREFIX, ALGORITHM_DH, ALGORITHM_PLAIN
+from secretstorage.dhcrypto import Session
 from secretstorage.exceptions import ItemNotFoundException, \
-	SecretServiceNotAvailableException
+ SecretServiceNotAvailableException
+from Crypto.Random.random import getrandbits
+from Crypto.Cipher.AES import AESCipher, MODE_CBC
+from secretstorage.dhcrypto import long_to_bytes, bytes_to_long
 
 class InterfaceWrapper(dbus.Interface):
 	"""Wraps :cls:`dbus.Interface` class and replaces some D-Bus exceptions
@@ -24,6 +30,9 @@ class InterfaceWrapper(dbus.Interface):
 					raise ItemNotFoundException('Item does not exist!')
 				if e.get_dbus_name() == DBUS_NO_SUCH_OBJECT:
 					raise ItemNotFoundException(e.get_dbus_message())
+				if e.get_dbus_name() in (DBUS_NO_REPLY, DBUS_NOT_SUPPORTED):
+					raise SecretServiceNotAvailableException(
+						e.get_dbus_message())
 				raise
 		return function_out
 
@@ -34,48 +43,83 @@ class InterfaceWrapper(dbus.Interface):
 		return result
 
 def bus_get_object(bus, name, object_path):
-	"""A wrapper around :func:`SessionBus.getObject` that raises
+	"""A wrapper around :meth:`SessionBus.get_object` that raises
 	:exc:`~secretstorage.exceptions.SecretServiceNotAvailableException`
 	when appropriate."""
 	try:
-		return bus.get_object(name, object_path)
+		return bus.get_object(name, object_path, introspect=False)
 	except dbus.exceptions.DBusException as e:
-		if e.get_dbus_name() in (DBUS_SERVICE_UNKNOWN, DBUS_EXEC_FAILED):
+		if e.get_dbus_name() in (DBUS_SERVICE_UNKNOWN, DBUS_EXEC_FAILED,
+		                         DBUS_NO_REPLY):
 			raise SecretServiceNotAvailableException(e.get_dbus_message())
 		raise
 
 def open_session(bus):
 	"""Returns a new Secret Service session."""
-	service_obj = bus.get_object(SECRETS, SS_PATH)
+	service_obj = bus_get_object(bus, SECRETS, SS_PATH)
 	service_iface = dbus.Interface(service_obj, SS_PREFIX+'Service')
-	return service_iface.OpenSession('plain', '', signature='sv')[1]
+	session = Session()
+	try:
+		output, result = service_iface.OpenSession(
+			ALGORITHM_DH,
+			dbus.ByteArray(long_to_bytes(session.my_public_key)),
+			signature='sv'
+		)
+	except dbus.exceptions.DBusException as e:
+		if e.get_dbus_name() != DBUS_NOT_SUPPORTED:
+			raise
+		output, result = service_iface.OpenSession(
+			ALGORITHM_PLAIN,
+			'',
+			signature='sv'
+		)
+		session.encrypted = False
+	else:
+		session.set_server_public_key(bytes_to_long(output))
+	session.object_path = result
+	return session
 
 def format_secret(session, secret, content_type):
 	"""Formats `secret` to make possible to pass it to the
 	Secret Service API."""
 	if not isinstance(secret, bytes):
 		secret = secret.encode('utf-8')
-	return dbus.Struct((session, '', dbus.ByteArray(secret), content_type))
+	if not session.encrypted:
+		return dbus.Struct((session.object_path, '',
+			dbus.ByteArray(secret), content_type))
+	# PKCS-7 style padding
+	padding = 0x10 - (len(secret) & 0xf)
+	secret += bytes(bytearray((padding,)) * padding)
+	aes_iv = long_to_bytes(getrandbits(0x80))
+	# If shorter than 16 bytes, prepend zero bytes
+	aes_iv = b'\x00' * (0x10 - len(aes_iv)) + aes_iv
+	aes_cipher = AESCipher(session.aes_key, mode=MODE_CBC, IV=aes_iv)
+	return dbus.Struct((
+		session.object_path,
+		dbus.Array(aes_iv),
+		dbus.Array(bytearray(aes_cipher.encrypt(secret))),
+		content_type
+	))
 
 def exec_prompt(bus, prompt, callback):
 	"""Executes the given `prompt`, when complete calls `callback`
 	function with two arguments: a boolean representing whether the
 	operation was dismissed and a list of unlocked item paths. A main
 	loop should be running and registered for this function to work."""
-	prompt_obj = bus.get_object(SECRETS, prompt)
+	prompt_obj = bus_get_object(bus, SECRETS, prompt)
 	prompt_iface = dbus.Interface(prompt_obj, SS_PREFIX+'Prompt')
-	prompt_iface.Prompt('')
+	prompt_iface.Prompt('', signature='s')
 	def new_callback(dismissed, unlocked):
 		if isinstance(unlocked, dbus.Array):
 			unlocked = list(unlocked)
 		callback(bool(dismissed), unlocked)
 	prompt_iface.connect_to_signal('Completed', new_callback)
 
-def exec_prompt_async_glib(bus, prompt):
-	"""Like :func:`exec_prompt`, but asynchronous (uses loop from GLib
+def exec_prompt_glib(bus, prompt):
+	"""Like :func:`exec_prompt`, but synchronous (uses loop from GLib
 	API). Returns (*dismissed*, *unlocked*) tuple."""
-	from gi.repository import GObject
-	loop = GObject.MainLoop()
+	from gi.repository import GLib
+	loop = GLib.MainLoop()
 	result = []
 	def callback(dismissed, unlocked):
 		result.append(dismissed)
@@ -85,20 +129,24 @@ def exec_prompt_async_glib(bus, prompt):
 	loop.run()
 	return result[0], result[1]
 
-def exec_prompt_async_qt(bus, prompt):
-	"""Like :func:`exec_prompt`, but asynchronous (uses loop from PyQt4
+def exec_prompt_qt(bus, prompt):
+	"""Like :func:`exec_prompt`, but synchronous (uses loop from PyQt5
 	API). Returns (*dismissed*, *unlocked*) tuple."""
-	from PyQt4.QtCore import QCoreApplication
+	from PyQt5.QtCore import QCoreApplication
 	app = QCoreApplication([])
 	result = []
 	def callback(dismissed, unlocked):
 		result.append(dismissed)
 		result.append(unlocked)
-		loop.quit()
+		app.quit()
 	exec_prompt(bus, prompt, callback)
 	app.exec_()
 	return result[0], result[1]
 
+# Compatibility aliases
+exec_prompt_async_glib = exec_prompt_glib
+exec_prompt_async_qt   = exec_prompt_qt
+
 def to_unicode(string):
 	"""Converts D-Bus string to unicode string."""
 	try:
